#include <PID_v1.h>

#define gMidpoint             50        //Mittelwert der PWM Ausgabe 
#define SAMPLE_PERIOD         5         //Milliseconds
#define DEFAULT_KP            0.4        //Default Kp, proportional gain parameter
#define DEFAULT_KD            0.02       //Default Kd, proportional gain parameter
#define DEFAULT_KI            0.0000001      //Default Ki, integral gain parameter

#define KP_INCREMENT          0.1        //Increment used for serial commands (gKp)
#define KD_INCREMENT          0.1        //Increment used for serial commands (gKd)
#define KI_INCREMENT          0.001     //Increment used for serial commands (gKi)
#define VALUE_INCREMENT       1          //Increment used for serial commands (gTargetValue)

//Pin Definitions
const int RPWM_Output   = 6; // Arduino PWM output pin 6; connect to IBT-2 pin 1 (RPWM) Timer 4 on the Mega (ATmega2560)
const int hallSensorPin = 0;
const int redLedPin     = 13;   
        
//PID Controller initialisation    
    double Output = 0;
    double Setpoint = 190;    //Default target hall effect readout
    double sens_value = 0;
    double y = 0.0;    // gelätter Sensorwert 
    double a = 0.7;   // exponential Filter Faktor
    PID mag_Controller(&y,&Output,&Setpoint, DEFAULT_KP, DEFAULT_KI, DEFAULT_KD,DIRECT);

void setup() {
   //int myEraser = 7;
   //TCCR4B &= ~myEraser;  // Timerregister fuer timer 4 loeschen
   //int myPrescaler = 2;  // Vorteiler definieren       
   //TCCR4B |= myPrescaler;  // Vorteiler laden
  // WGM20 is used for Phase Correct PWM, COM2A1/COM2B1 sets output to non-inverted
    TCCR4A = 0;
    TCCR4A = _BV(COM2A1) | _BV(COM2B1) | _BV(WGM20);
    // PWM frequency is 16MHz/255/2/<prescaler>, prescaler is 1 here by using CS20
    TCCR4B = 0;
    TCCR4B = _BV(CS20);
  
  pinMode(RPWM_Output, OUTPUT);
  pinMode(redLedPin,   OUTPUT);
  analogWrite(RPWM_Output, gMidpoint); // Magnetfeld aufbauen
  analogWrite(redLedPin, 0);
  pinMode(hallSensorPin,INPUT);    
  Serial.begin(115200); 
  //PID Settings
     mag_Controller.SetMode(AUTOMATIC);
     mag_Controller.SetOutputLimits(-50, 150); 
     mag_Controller.SetSampleTime(SAMPLE_PERIOD ); 
     mag_Controller.SetTunings(DEFAULT_KP,DEFAULT_KI,DEFAULT_KD);
}

void loop() {
  y = a * y + (1.0 - a) * analogRead(hallSensorPin);
  mag_Controller.Compute();
  analogWrite(RPWM_Output,abs(Output + gMidpoint)); // Output wird immer um Offset angehoben
  Serial.println(Output);
 } 
 //End Loop
 
